Class: Protocol::HPACK::Context
- Inherits:
-
Object
- Object
- Protocol::HPACK::Context
- Defined in:
- lib/protocol/hpack/context.rb
Overview
To decompress header blocks, a decoder only needs to maintain a dynamic table as a decoding context. No other state information is needed.
Constant Summary collapse
- STATIC_TABLE =
[ [':authority', ''], [':method', 'GET'], [':method', 'POST'], [':path', '/'], [':path', '/index.html'], [':scheme', 'http'], [':scheme', 'https'], [':status', '200'], [':status', '204'], [':status', '206'], [':status', '304'], [':status', '400'], [':status', '404'], [':status', '500'], ['accept-charset', ''], ['accept-encoding', 'gzip, deflate'], ['accept-language', ''], ['accept-ranges', ''], ['accept', ''], ['access-control-allow-origin', ''], ['age', ''], ['allow', ''], ['authorization', ''], ['cache-control', ''], ['content-disposition', ''], ['content-encoding', ''], ['content-language', ''], ['content-length', ''], ['content-location', ''], ['content-range', ''], ['content-type', ''], ['cookie', ''], ['date', ''], ['etag', ''], ['expect', ''], ['expires', ''], ['from', ''], ['host', ''], ['if-match', ''], ['if-modified-since', ''], ['if-none-match', ''], ['if-range', ''], ['if-unmodified-since', ''], ['last-modified', ''], ['link', ''], ['location', ''], ['max-forwards', ''], ['proxy-authenticate', ''], ['proxy-authorization', ''], ['range', ''], ['referer', ''], ['refresh', ''], ['retry-after', ''], ['server', ''], ['set-cookie', ''], ['strict-transport-security', ''], ['transfer-encoding', ''], ['user-agent', ''], ['vary', ''], ['via', ''], ['www-authenticate', ''], ].each {|pair| pair.each(&:freeze).freeze}.freeze
Instance Attribute Summary collapse
-
#huffman ⇒ Object
readonly
Returns the value of attribute huffman.
-
#index ⇒ Object
readonly
Returns the value of attribute index.
-
#table ⇒ Object
readonly
Current table of header key-value pairs.
-
#table_size ⇒ Object
Returns the value of attribute table_size.
Instance Method Summary collapse
-
#add_command(*header) ⇒ Hash
Emits command for a header.
-
#current_table_size ⇒ Integer
Returns current table size in octets.
-
#decode(command) ⇒ Array
Header Block Processing - tools.ietf.org/html/draft-ietf-httpbis-header-compression-10#section-4.1.
-
#dereference(index) ⇒ Array
Finds an entry in current dynamic table by index.
-
#encode(headers) ⇒ Array
Plan header compression according to @index :never Do not use dynamic table or static table reference at all.
-
#initialize(table = nil, huffman: :shorter, index: :all, table_size: 4096) ⇒ Context
constructor
Initializes compression context with appropriate client/server defaults and maximum size of the dynamic table.
- #initialize_copy(other) ⇒ Object
Constructor Details
#initialize(table = nil, huffman: :shorter, index: :all, table_size: 4096) ⇒ Context
Initializes compression context with appropriate client/server defaults and maximum size of the dynamic table.
116 117 118 119 120 121 122 123 |
# File 'lib/protocol/hpack/context.rb', line 116 def initialize(table = nil, huffman: :shorter, index: :all, table_size: 4096) @huffman = huffman @index = index @table_size = table_size @table = (table&.dup) || [] end |
Instance Attribute Details
#huffman ⇒ Object (readonly)
Returns the value of attribute huffman.
135 136 137 |
# File 'lib/protocol/hpack/context.rb', line 135 def huffman @huffman end |
#index ⇒ Object (readonly)
Returns the value of attribute index.
136 137 138 |
# File 'lib/protocol/hpack/context.rb', line 136 def index @index end |
#table ⇒ Object (readonly)
Current table of header key-value pairs.
133 134 135 |
# File 'lib/protocol/hpack/context.rb', line 133 def table @table end |
#table_size ⇒ Object
Returns the value of attribute table_size.
138 139 140 |
# File 'lib/protocol/hpack/context.rb', line 138 def table_size @table_size end |
Instance Method Details
#add_command(*header) ⇒ Hash
Emits command for a header. Prefer static table over dynamic table. Prefer exact match over name-only match.
@index controls whether to use the dynamic table, static table, or both.
:never Do not use dynamic table or static table reference at all.
:static Use static table only.
:all Use all of them.
248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 |
# File 'lib/protocol/hpack/context.rb', line 248 def add_command(*header) exact = nil name_only = nil if [:all, :static].include?(@index) STATIC_TABLE.each_index do |i| if STATIC_TABLE[i] == header exact ||= i break elsif STATIC_TABLE[i].first == header.first name_only ||= i end end end if [:all].include?(@index) && !exact @table.each_index do |i| if @table[i] == header exact ||= i + STATIC_TABLE.size break elsif @table[i].first == header.first name_only ||= i + STATIC_TABLE.size end end end if exact {name: exact, type: :indexed} elsif name_only {name: name_only, value: header.last, type: :incremental} else {name: header.first, value: header.last, type: :incremental} end end |
#current_table_size ⇒ Integer
Returns current table size in octets
291 292 293 |
# File 'lib/protocol/hpack/context.rb', line 291 def current_table_size @table.inject(0) {|r, (k, v)| r + k.bytesize + v.bytesize + 32} end |
#decode(command) ⇒ Array
Header Block Processing
164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 |
# File 'lib/protocol/hpack/context.rb', line 164 def decode(command) emit = nil case command[:type] when :change_table_size self.table_size = command[:value] when :indexed # Indexed Representation # An _indexed representation_ entails the following actions: # o The header field corresponding to the referenced entry in either # the static table or dynamic table is added to the decoded header # list. idx = command[:name] k, v = dereference(idx) emit = [k, v] when :incremental, :no_index, :never_indexed # A _literal representation_ that is _not added_ to the dynamic table # entails the following action: # o The header field is added to the decoded header list. # A _literal representation_ that is _added_ to the dynamic table # entails the following actions: # o The header field is added to the decoded header list. # o The header field is inserted at the beginning of the dynamic table. if command[:name].is_a? Integer k, v = dereference(command[:name]) command = command.dup command[:index] ||= command[:name] command[:value] ||= v command[:name] = k end emit = [command[:name], command[:value]] add_to_table(emit) if command[:type] == :incremental else raise CompressionError, "Invalid type: #{command[:type]}" end return emit end |
#dereference(index) ⇒ Array
Finds an entry in current dynamic table by index. Note that index is zero-based in this module.
If the index is greater than the last index in the static table, an entry in the dynamic table is dereferenced.
If the index is greater than the last header index, an error is raised.
150 151 152 153 154 155 156 157 |
# File 'lib/protocol/hpack/context.rb', line 150 def dereference(index) # NOTE: index is zero-based in this module. value = STATIC_TABLE[index] || @table[index - STATIC_TABLE.size] raise CompressionError, "Index #{index} too large" unless value return value end |
#encode(headers) ⇒ Array
Plan header compression according to @index
:never Do not use dynamic table or static table reference at all.
:static Use static table only.
:all Use all of them.
219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 |
# File 'lib/protocol/hpack/context.rb', line 219 def encode(headers) commands = [] # Literals commands are marked with :no_index when index is not used no_index = [:static, :never].include?(@index) headers.each do |field, value| command = add_command(field, value) command[:type] = :no_index if no_index && command[:type] == :incremental commands << command decode(command) end return commands end |
#initialize_copy(other) ⇒ Object
125 126 127 128 129 130 |
# File 'lib/protocol/hpack/context.rb', line 125 def initialize_copy(other) super # This is the only mutable state: @table = @table.dup end |